
const St = imports.gi.St;
const Shell = imports.gi.Shell;
const Main = imports.ui.main;
const Tweener = imports.ui.tweener;
const AltTab = imports.ui.altTab;
const Dash = imports.ui.dash;
const Lang = imports.lang;
const Gdk = imports.gi.Gdk;
const GnomeDesktop = imports.gi.GnomeDesktop;
const LD = imports.gi.LunarDate;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

let replacementFunc = [], ld = new LD.Date, settingsChanged = [], localSettings = {};
let settings;

ld.setDate = function (date) {
  this.set_solar_date(date.getFullYear(), (1+date.getMonth()), date.getDate(), date.getHours());
};

ld.setDateNoon = function (date) {
  this.set_solar_date(date.getFullYear(), (1+date.getMonth()), date.getDate(), 12);
};

function init() {
}

function _make_new_with_args (my_class, args) {
  return new (Function.prototype.bind.apply(my_class,
					    [null].concat(Array.prototype.slice.call(args))))();
}

function enable() {
  settings = Convenience.getSettings();
  settings.connect('changed', function() {
    for (x in settingsChanged) {
      settingsChanged[x]();
    }
  });

  let dm = Main.panel.statusArea.dateMenu;

  let cal = dm._calendar;
  let ev = dm._eventList;
  let cny_now;

  replacementFunc.calendarUpdater = cal._update;
  replacementFunc.originalMonthHeader = cal._headerFormat;
  replacementFunc.evAddPeriod = ev._addPeriod;
  if (cal._rebuildCalendar !== undefined) // gnome 3.12
    replacementFunc.calendarRebuilder = cal._rebuildCalendar;

  let fixupHeader = _("%A %B %e, %Y").match(/%Y[^%]+%/);
  if (fixupHeader) {
    cal._headerFormat = cal._headerFormat.replace(/%Y.%/, fixupHeader);
  }

  dm._clock.run_dispose();
  dm._clock = new GnomeDesktop.WallClock();

  let replacementUpdateClockAndDate = function () {
    dm._updateClockAndDate.apply(dm, arguments);
    ld.setDate(new Date());
    cny_now = ld.strftime("%(shengxiao)");

    // button in gnome 3.12
    let date_label = dm._date.text !== undefined ? dm._date : dm._date.get_first_child();
    date_label.text = date_label.text + (date_label.text.match(/[.,]/) ? ", " : "\u2001") +
      cny_now + "\n" + ld.strftime("%(NIAN)年%(YUE)月%(RI)日");

    let show_date = settings.get_boolean('show-date');
    dm._clockDisplay.text = dm._clockDisplay.text +
      (show_date ? "\u2001" + ld.strftime("%(YUE)月%(RI)日") : "") +
      (settings.get_boolean('show-time') ? ld.strftime(show_date ? "%(SHI)时" : "%(SHI)") : "");
  };

  dm._clock.connect('notify::clock', Lang.bind(dm, replacementUpdateClockAndDate));
  replacementUpdateClockAndDate();

  let _toHex = function (number) {
    let t = (number * 65535).toString(16);
    while (t.length < 4) t = '0' + t;
    return t;
  };

  settingsChanged.updateLocal = function () {
    let color = new Gdk.RGBA;
    color.parse('white');
    color.parse(settings.get_string('jieri-color'));
    localSettings.jieri_color = "#" + _toHex(color.red) + _toHex(color.green) + _toHex(color.blue);
  };
  settingsChanged.updateLocal();

  settingsChanged.refreshClock = replacementUpdateClockAndDate;

  let lunarButton = function (orig_button, iter_date, oargs) {
    let fat_button = false;
    if (oargs[0].label == iter_date.getDate().toString()) {
      iter_date._lunar_iter_found = true;
      ld.setDateNoon(iter_date);
      let yd = settings.get_boolean('show-calendar') ? ld.strftime("%(ri)") : "";
      let jr = settings.get_boolean('jieri') ? ld.strftime("%(jieri)") : "";
      let dx = settings.get_string('zti-dx');
      fat_button = yd != "";
      let l = oargs[0].label;
      if (yd != "") l = "<u> " + l + " </u>"; 
      if (jr != "") l = "<span weight='bold' color='" + localSettings.jieri_color + "'>" + l + "</span>";
      if (yd != "") l += "\n<small>" + 
	ld.strftime(yd == "1" ? "%(YUE)月" : "%(RI)") +
	"</small>";
      if (dx != "none") l = "<span size='" + dx + "'>" + l + "</span>";
      oargs[0].label = l;
    }
    let new_button = _make_new_with_args(orig_button, oargs);
    if (fat_button) {
      new_button.set_width(cal._digitWidth * 5);
    }
    return new_button;
  };

  let updateYear = function (that) {
    ld.setDateNoon(that._selectedDate);
    let cny = ld.strftime("%(shengxiao)");
    if (cny != cny_now) {
      that._monthLabel.text = that._monthLabel.text + " / " + cny;
    }
  };

  if (cal._rebuildCalendar !== undefined) { // gnome 3.12
    cal._rebuildCalendar = function () {
      let orig_button = St.Button;
      let orig_date = Date;
      let iter_date = new orig_date();

      Date = function () {
	let new_date = _make_new_with_args(orig_date, arguments);
	if (!iter_date._lunar_iter_found &&
	    arguments.length > 0 && arguments[0] instanceof orig_date) {
	  iter_date = new_date;
	}
	return new_date;
      };

      St.Button = function () {
	return lunarButton(orig_button, iter_date, arguments);
      };

      replacementFunc.calendarRebuilder.apply(this, arguments);

      St.Button = orig_button;
      Date = orig_date;
    };

    cal._update = function () {
      replacementFunc.calendarUpdater.apply(this, arguments);
      updateYear(this);
    };

    settingsChanged.rebuildCal = function () {
      cal._rebuildCalendar();
      ev._update();
    };
    settingsChanged.rebuildCal();
  }
  else { // gnome <= 3.10
    cal._update = function () {
      let orig_button = St.Button;
      let orig_date = Date;
      let iter_date = new orig_date();

      Date = function () {
	let new_date = _make_new_with_args(orig_date, arguments);
	if (arguments.length > 0 && arguments[0] instanceof orig_date) {
	  iter_date = new_date;
	}
	return new_date;
      };

      St.Button = function () {
	return lunarButton(orig_button, iter_date, arguments);
      };

      replacementFunc.calendarUpdater.apply(this, arguments);
      updateYear(this);

      St.Button = orig_button;
      Date = orig_date;
    };
  }

  ev._addPeriod = function () {
    let start_begin = 1; // will be > 1 for gnome 3.10
    while (start_begin < arguments.length && !(
      arguments[start_begin] instanceof Date)) { start_begin++; }
    let one_day = arguments[start_begin].getDate() == arguments[start_begin + 1].getDate();
    let tomorrow = arguments[0] == _("Tomorrow");
    let named_day = tomorrow || arguments[0] == _("Today");

    if (one_day) {
      ld.setDateNoon(arguments[start_begin]);

      if (!named_day) {
	let cny = ld.strftime("%(shengxiao)");
	arguments[0] = arguments[0] + (start_begin > 1 ? "\u2001\u2001\u2001\u2001" : "\u2001") +
	  ld.strftime((cny != cny_now ? "%(NIAN)年" : "") + "%(YUE)月%(RI)日");
      }
    }

    let orig_box = St.BoxLayout;
    let capturedBoxes = []; // gnome <= 3.8
    St.BoxLayout = function () {
      let new_box = _make_new_with_args(orig_box, arguments);
      if (arguments.length && arguments[0].style_class) {
	capturedBoxes[arguments[0].style_class] = new_box;
      }
      return new_box;
    };

    let idx = replacementFunc.evAddPeriod.apply(this, arguments);

    St.BoxLayout = orig_box;
    let that = this;

    let addEvent = idx === undefined ? function (l1, l2) {
      that._addEvent(null, capturedBoxes['events-time-box'], capturedBoxes['events-event-box'], false, null, l1, l2);
    } : function (l1, l2) { // gnome 3.10 case
      let origFormatET = imports.ui.calendar._formatEventTime;
      imports.ui.calendar._formatEventTime = function () { return l1; };
      that._addEvent({ summary: l2 }, idx, false);
      imports.ui.calendar._formatEventTime = origFormatET;
      idx++;
    };

    if (one_day) {
      if (!tomorrow) {
	if (settings.get_boolean('gen-zhi')) {
	  addEvent("干支", ld.strftime("%(Y60)年%(M60)月%(D60)日"));
	}
	if (settings.get_boolean('ba-zi')) {
	  addEvent("八字", ld.strftime("%(Y8)年%(M8)月%(D8)日"));
	}
      }

      let jr = settings.get_boolean('jieri') ? ld.strftime("%(jieri)") : "";
      if (jr != "") {
	addEvent("节日", ld.get_jieri("\n"));
      }
    }

    if (idx !== undefined) // gnome 3.10
      return idx + 1;
  };
}

function disable() {
  let dm = Main.panel.statusArea.dateMenu;

  dm._calendar._update = replacementFunc.calendarUpdater;
  dm._calendar._headerFormat = replacementFunc.originalMonthHeader;
  dm._eventList._addPeriod = replacementFunc.evAddPeriod;
  dm._calendar._rebuildCalendar = replacementFunc.calendarRebuilder;

  dm._clock.run_dispose();
  dm._clock = new GnomeDesktop.WallClock();
  dm._clock.connect('notify::clock', Lang.bind(dm, dm._updateClockAndDate));
  dm._updateClockAndDate();

  settingsChanged = [];
  settings.run_dispose();
  settings = null;

  delete replacementFunc.calendarUpdater;
  delete replacementFunc.originalMonthHeader;
  delete replacementFunc.evAddPeriod;
  delete replacementFunc.calendarRebuilder;
}
